unit UfrmMain;

interface
{$WARN UNIT_PLATFORM OFF}
uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls,
  Vcl.ComCtrls, System.IniFiles, System.Math, Vcl.FileCtrl, ShxFont;
{$WARN UNIT_PLATFORM ON}
type
  TfrmMain = class(TForm)
    memGlyphDef: TMemo;
    cbShapeNum: TComboBox;
    cbFontNames: TComboBox;
    btnExportShp: TButton;
    edMaxStrLen: TEdit;
    lblMaxStrLen: TLabel;
    edFontsDirPath: TEdit;
    btnSelFontsDirPath: TButton;
    memFontInfo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure cbFontNamesChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnExportShpClick(Sender: TObject);
    procedure btnSelFontsDirPathClick(Sender: TObject);
    procedure cbShapeNumChange(Sender: TObject);
  private
    FFontsDirPath: String;
    FFontFileName: String;
    FShapeIndex: Integer;
    procedure PopulateShxFontNames;
    procedure LoadFont;
    procedure SetFontShxParams;
    procedure SaveSettings;
    procedure LoadSettings;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

resourcestring
  rsDefFontsPath = 'D:\';
  rsDefFontFileName = '';

procedure EnumShxFontNames(const APath: String; AStrings: TStrings);
var
  sr: TSearchRec;
begin
  AStrings.BeginUpdate;
  try
    AStrings.Clear;
    if FindFirst(APath + '*.shx', faAnyFile, sr) = 0 then
    begin
      repeat
        AStrings.Add(sr.Name);
      until FindNext(sr) <> 0;
    end;
  finally
    AStrings.EndUpdate;
  end;
end;

function GetDigitsCount(ANum: Word): Byte;
begin
  if ANum < 10 then
    Result:= 1
  else if ANum < 100 then
    Result:= 2
  else if ANum < 1000 then
    Result:= 3
  else if ANum < 10000 then
    Result:= 4
  else
    Result:= 5;
end;

procedure TfrmMain.PopulateShxFontNames;
var
  i: Integer;
begin
  EnumShxFontNames(FFontsDirPath, cbFontNames.Items);
  if cbFontNames.Items.Count > 0 then
  begin
    i:= cbFontNames.Items.IndexOf(FFontFileName);
    if i > -1 then
      cbFontNames.ItemIndex:= i
    else
      cbFontNames.ItemIndex:= 0;
  end;
//  cbFontNames.OnChange(cbFontNames);
end;

procedure TfrmMain.SaveSettings;
var
  Ini: TIniFile;
begin
  SetFontShxParams;
  Ini:= TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  try
    Ini.WriteString('Main', 'FontsDirPath', FFontsDirPath);
    Ini.WriteString('Main', 'FontFileName', FFontFileName);
    Ini.WriteInteger('Main', 'ShapeIndex', cbShapeNum.ItemIndex);
    Ini.WriteInteger('Main', 'MaxStrLen', FontShx.MaxStrLen);
  finally
    FreeAndNil(Ini);
  end;
end;

procedure TfrmMain.SetFontShxParams;
begin
  FontShx.MaxStrLen:= StrToIntDef(edMaxStrLen.Text, FontShx.MaxStrLen);
  edMaxStrLen.Text:= FormatFloat('0', FontShx.MaxStrLen);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Caption:= Application.Title;
  FFontsDirPath:= rsDefFontsPath;
  FFontFileName:= rsDefFontFileName;
  LoadSettings;
  edFontsDirPath.Text:= FFontsDirPath;
  PopulateShxFontNames;
  LoadFont;
  if InRange(FShapeIndex, 0, cbShapeNum.Items.Count - 1) then
    cbShapeNum.ItemIndex:= FShapeIndex;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  SaveSettings;
end;

procedure TfrmMain.LoadFont;
var
  AFileName: String;
  i, n, m: Integer;
begin
  AFileName:= FFontsDirPath + FFontFileName;
  if (FFontFileName <> '') and FontShx.LoadFromFile(AFileName) then
  begin
    cbShapeNum.Items.BeginUpdate;
    try
      cbShapeNum.Items.Clear;
      n:= GetDigitsCount(FontShx.Count - 1);
      m:= GetDigitsCount(FontShx[FontShx.Count - 1].Number);
      for i:= 0 to FontShx.Count - 1 do
        cbShapeNum.Items.Add(Format('%*d %s[%*d 0x%.4x] %s', [n, i,
          Chr(FontShx[i].Number), m, FontShx[i].Number, FontShx[i].Number, FontShx[i].Name]));
    finally
      cbShapeNum.Items.EndUpdate;
    end;
    memFontInfo.Lines.Clear;
    if FontShx.IsFont then
      if FontShx.FontName <> '' then
        memFontInfo.Lines.Add(Format('FontName: %s', [FontShx.FontName]))
      else
        memFontInfo.Lines.Add('FontName')
    else
      memFontInfo.Lines.Add(' ');
    memFontInfo.Lines.Add(Format('FontKind: %s', [ShxFontKindToStr(FontShx.FontKind)]));
    Caption:= Format('%s - %s', [Application.Title, FontShx.FontFileName]);
  end else
  begin
    memFontInfo.Lines.Add('FontName');
    memFontInfo.Lines.Add('FontKind');
  end;
  memFontInfo.Lines.Add(Format('Above: %d', [FontShx.Above]));
  memFontInfo.Lines.Add(Format('Below: %d', [FontShx.Below]));
  memFontInfo.Lines.Add(Format('Modes: %d', [FontShx.Modes]));
  if FontShx.FontKind = fkUniFont then
  begin
    memFontInfo.Lines.Add(Format('Encoding: %d', [FontShx.UniEncoding]));
    memFontInfo.Lines.Add(Format('Type: %d', [FontShx.UniType]));
  end;
end;

procedure TfrmMain.LoadSettings;
var
  Ini: TIniFile;
begin
  Ini:= TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  try
    FFontsDirPath:= Ini.ReadString('Main', 'FontsDirPath', FFontsDirPath);
    FFontFileName:= Ini.ReadString('Main', 'FontFileName', FFontFileName);
    FShapeIndex:= Ini.ReadInteger('Main', 'ShapeIndex', 0);
    edMaxStrLen.Text:= FormatFloat('0', Ini.ReadInteger('Main', 'MaxStrLen', FontShx.MaxStrLen));
  finally
    FreeAndNil(Ini);
  end;
end;

procedure TfrmMain.cbFontNamesChange(Sender: TObject);
begin
  if cbFontNames.ItemIndex < 0 then
    FFontFileName:= ''
  else
    FFontFileName:= cbFontNames.Items[cbFontNames.ItemIndex];
  LoadFont;
end;

procedure TfrmMain.cbShapeNumChange(Sender: TObject);
begin
  if cbShapeNum.ItemIndex < 0 then Exit;
  SetFontShxParams;
  memGlyphDef.Lines.Text:= FontShx[cbShapeNum.ItemIndex].GetGlyphDefinition(FontShx.MaxStrLen);
end;

procedure TfrmMain.btnExportShpClick(Sender: TObject);
begin
  if FontShx.FontFileName = '' then Exit;
  with TSaveDialog.Create(Self) do
  begin
    Filter:= '   AutoCAD (*.shp)|*.shp';
    DefaultExt:= 'shp';
    FileName:= ExtractFileName(ChangeFileExt(FontShx.FontFileName, '.shp'));
    InitialDir:= ExtractFilePath(FontShx.FontFileName);
    Options:= [ofOverwritePrompt, ofNoReadOnlyReturn, ofPathMustExist, ofHideReadOnly, ofEnableSizing];
    if not Execute then Exit;
    SetFontShxParams;
    FontShx.ExportToShpFile(FileName);
  end;
end;

procedure TfrmMain.btnSelFontsDirPathClick(Sender: TObject);
begin
  if not Vcl.FileCtrl.SelectDirectory(' :', '', FFontsDirPath, [sdNewUI], Self) then Exit;
  FFontsDirPath:= IncludeTrailingPathDelimiter(FFontsDirPath);
  edFontsDirPath.Text:= FFontsDirPath;
  PopulateShxFontNames;
  LoadFont;
end;

end.
